/*
**  This example illustrates how to estimate garch models
**  using the procedures rather than the keyword commands.
**  This allows you to provide nonstandard constraints
**  or other conditions to the model.  In this example
**  the Nelson & Cao constraints are compared with the
**  traditional constraints.
*/


   library fanpac;
   fanset;    /* resets globals to default values */
              /* when the command file is re-run  */


   output file = fanpac3.out reset;

   _fan_p = 1;   /*  garch(1,3) model */
   _fan_q = 2;

   _fan_series = loadd("example");

   nobs = rows(_fan_Series);


   start = { .2,    /* omega */
             .3,    /* garch_1 */
             .2,    /* arch_1 */  
             .1,    /* arch_2 */
              1 };  /* constant */

    _nlp_GradProc = &garch_n_grd;  /* gradient proc */


   /*
   **  Nelson and Cao constraints
   */

   _nlp_IneqProc = &garch_ineq;  /* inequality constraints */

   _nlp_Bounds = { .001   1e250,  /* omega > 0    */
                    0     1,      /* garch_1 >= 0 */
                   -1e250 1e250,   
                   -1e250 1e250,   
                   -1e250 1e250 };

   { coefs,fct,grad,rcode } = nlp(&garch_n,start);    

   print;
   print;
   print "       Nelson & Cao constraints";
   print;

   if rcode <= 2;   
       /* covariance matrix of parameters */
       h = _nlp_CovPar(coefs,&garch_n,&garch_n_grd,nobs,1);
   else;
       h = error(0);
   endif;           

   format /rd 12,4;
   if not scalmiss(h);
       alpha = .05;  /* 95% cl's */

       dv = cdftci(0.5*alpha,nobs-rows(coefs))*sqrt(diag(h));
       print "confidence limits from standard errors";
       print;
       print "  Coefficients    lower cl    upper cl";
       print coefs~(coefs-dv)~(coefs+dv);
       print;

       /* confidence limits by inversion of Wald statistic   */
       sel = 0;      /* selection vector, if zero, all cl's computed */
       cl = _nlp_climits(coefs,h,nobs,alpha,sel);
   endif;

   format /rd 12,4;
   if not scalmiss(cl) and not scalmiss(h);

       print "confidence limits from inversion of Wald statistic";
       print;
       print "  Coefficients    lower cl    upper cl";
       print coefs~cl;

   else;

       print "  Coefficients";
       print coefs;

   endif; 


   /*
   ** standard constraints
   */


   _nlp_IneqProc = {.};

   _nlp_c = { 0 -1 -1 -1 0 };   /* garch + arch < 1 */
   _nlp_d = { -1 };

   _nlp_Bounds = { .001   1, /* omega > 0    */
                    0     1,     /* garch_1 >= 0 */
                    0     1,     /* arch_1 >= 0  */ 
                    0     1,     /* arch_2 >= 0  */ 
                   -1e250 1e250 };



   { coefs,fct,grad,rcode } = nlp(&garch_n,start);    

   print;
   print;
   print "       standard constraints";

   if rcode <= 2;   
       /* covariance matrix of parameters */
       h = _nlp_CovPar(coefs,&garch_n,&garch_n_grd,nobs,1);
   else;
       h = error(0);
   endif;           


   format /rd 12,4;
   if not scalmiss(h);
       alpha = .05;  /* 95% cl's */

       dv = cdftci(0.5*alpha,nobs-rows(coefs))*sqrt(diag(h));
       print;
       print "confidence limits from standard errors";
       print;
       print "  Coefficients    lower cl    upper cl";
       print coefs~(coefs-dv)~(coefs+dv);
       print;

       /* confidence limits by inversion of Wald statistic   */
       sel = 0;      /* selection vector, if zero, all cl's computed */
       cl = _nlp_climits(coefs,h,nobs,alpha,sel);
   endif;

   format /rd 12,4;
   if not scalmiss(cl) and not scalmiss(h);
       print;
       print "confidence limits from inversion of Wald statistic";
       print "  Coefficients    lower cl    upper cl";
       print coefs~cl;

   else;

       print "  Coefficients";
       print coefs;

   endif; 

   output off;


